import uuid
from typing import Dict

from jeffy.handlers.common import CommonHandlerMixin
from jeffy.handlers.rest_api import RestApiHandlerMixin
from jeffy.handlers.s3 import S3HandlerMixin
from jeffy.handlers.schedule import ScheduleHandlerMixin
from jeffy.handlers.sns import SnsHandlerMixin
from jeffy.handlers.sqs import SqsHandlerMixin
from jeffy.handlers.streams import StreamsHandlerMixin


class Handlers(
    CommonHandlerMixin,
    RestApiHandlerMixin,
    S3HandlerMixin,
    ScheduleHandlerMixin,
    SnsHandlerMixin,
    SqsHandlerMixin,
    StreamsHandlerMixin
):
    """Jeffy event handler decorators."""

    def __init__(self, app) -> None:    # type: ignore
        self.app = app

    def capture_correlation_id(self, payload: Dict = {}) -> str:
        """
        Automatically generates and capurures the correlation ID.

        Parameters
        ----------
        payload: dict
            Payload event with including correlation attribute
        Returns
        -------
        correlation_id : str
        """
        correlation_id = ''
        if self.app.correlation_attr_name in payload:                               # type: ignore
            correlation_id = payload[self.app.correlation_attr_name]                # type: ignore
        elif self.app.correlation_id_header in payload.get('headers', {}):          # type: ignore
            correlation_id = payload['headers'][self.app.correlation_id_header]     # type: ignore
        else:
            correlation_id = str(uuid.uuid4())
        self.app.logger.update_context({self.app.correlation_attr_name: correlation_id})  # type: ignore
        self.app.correlation_id = correlation_id  # type: ignore
        return correlation_id



import json
from typing import Any, Dict

import boto3

import botocore

from jeffy.sdk import SdkBase


class Sqs(SdkBase):
    """SQS Client."""

    _resource = None

    @classmethod
    def get_resource(self) -> botocore.client.BaseClient:
        """
        Get boto3 client for SQS.

        Usage::
            >>> from jeffy.sdk.sqs import Sqs
            >>> Sqs.get_resource().send_message(...)
        """
        if Sqs._resource is None:
            Sqs._resource = boto3.client('sqs')
        return Sqs._resource

    def send_message(
        self,
        message: Any,
        queue_url: str,
        correlation_id: str = ''
    ) -> Dict:
        """
        Send message to SQS Queue with correlationid.

        Usage::
            >>> from jeffy.sdk.sqs import Sqs
            >>> Sqs().send_message(...)
        """
        if correlation_id == '':
            correlation_id = self.app.correlation_id
        return self.get_resource().send_message(
            QueueUrl=queue_url,
            MessageBody=json.dumps({
                self.app.correlation_attr_name: correlation_id,
                'item': message
            })
        )
